-
Notifications
You must be signed in to change notification settings - Fork 842
[SYCL][DOC] Spec for discard queue events #4922
New issue
Have a question about this project? Sign up for a free GitHub account to open an issue and contact its maintainers and the community.
By clicking “Sign up for GitHub”, you agree to our terms of service and privacy statement. We’ll occasionally send you account related emails.
Already on GitHub? Sign in to your account
Merged
bader
merged 13 commits into
intel:sycl
from
alexanderfle:spec_for_avoid_event_creation
Nov 23, 2021
Merged
Changes from 9 commits
Commits
Show all changes
13 commits
Select commit
Hold shift + click to select a range
d3e543e
[SYCL][DOC] Add avoid_event_creation extension
alexanderfle 95ff7fb
Update avoid_event_creation.asciidoc
alexanderfle 97f2913
Update avoid_event_creation.asciidoc
alexanderfle 6788b91
Fix review comments #1
alexanderfle 1fdf67d
Fix review comments 2
alexanderfle 6d8bc90
Minor fix
alexanderfle 226421f
Fix review comments 3
alexanderfle e6f1871
Fix review comments 4
alexanderfle 5962802
Update SYCL_EXT_ONEAPI_DISCARD_QUEUE_EVENTS.asciidoc
alexanderfle 488e19b
Update sycl/doc/extensions/DiscardQueueEvents/SYCL_EXT_ONEAPI_DISCARD…
alexanderfle 49679e4
Update SYCL_EXT_ONEAPI_DISCARD_QUEUE_EVENTS.asciidoc
alexanderfle 414f578
Added words about L0 support
alexanderfle bf1cff8
Update sycl/doc/extensions/DiscardQueueEvents/SYCL_EXT_ONEAPI_DISCARD…
alexanderfle File filter
Filter by extension
Conversations
Failed to load comments.
Loading
Jump to
Jump to file
Failed to load files.
Loading
Diff view
Diff view
There are no files selected for viewing
187 changes: 187 additions & 0 deletions
187
...doc/extensions/DiscardQueueEvents/SYCL_EXT_ONEAPI_DISCARD_QUEUE_EVENTS.asciidoc
This file contains hidden or bidirectional Unicode text that may be interpreted or compiled differently than what appears below. To review, open the file in an editor that reveals hidden Unicode characters.
Learn more about bidirectional Unicode characters
| Original file line number | Diff line number | Diff line change |
|---|---|---|
| @@ -0,0 +1,187 @@ | ||
| = SYCL_EXT_ONEAPI_DISCARD_QUEUE_EVENTS | ||
| :source-highlighter: coderay | ||
| :coderay-linenums-mode: table | ||
|
|
||
| // This section needs to be after the document title. | ||
| :doctype: book | ||
| :toc2: | ||
| :toc: left | ||
| :encoding: utf-8 | ||
| :lang: en | ||
|
|
||
| :blank: pass:[ +] | ||
|
|
||
| // Set the default source code type in this document to C++, | ||
| // for syntax highlighting purposes. This is needed because | ||
| // docbook uses c++ and html5 uses cpp. | ||
| :language: {basebackend@docbook:c++:cpp} | ||
|
|
||
| // This is necessary for asciidoc, but not for asciidoctor | ||
| :cpp: C++ | ||
|
|
||
| == Introduction | ||
|
|
||
| IMPORTANT: This specification is a draft. | ||
|
|
||
| NOTE: Khronos(R) is a registered trademark and SYCL(TM) and SPIR(TM) are | ||
| trademarks of The Khronos Group Inc. OpenCL(TM) is a trademark of Apple Inc. | ||
| used by permission by Khronos. | ||
|
|
||
| This document describes an extension that introduces a discard_events property for | ||
| SYCL queues. This property enables developers to inform a SYCL implementation that | ||
| the returned event can be discarded. | ||
|
alexanderfle marked this conversation as resolved.
Outdated
|
||
|
|
||
| == Notice | ||
|
|
||
| Copyright (c) 2021 Intel Corporation. All rights reserved. | ||
|
|
||
| == Status | ||
|
|
||
| Working Draft | ||
|
|
||
| This is a preview extension specification, intended to provide early access to | ||
| a feature for review and community feedback. When the feature matures, this | ||
| specification may be released as a formal extension. | ||
|
|
||
| Because the interfaces defined by this specification are not final and are | ||
| subject to change they are not intended to be used by shipping software | ||
| products. | ||
|
|
||
| == Version | ||
|
|
||
| Built On: {docdate} + | ||
|
alexanderfle marked this conversation as resolved.
Outdated
|
||
| Revision: 1 | ||
|
|
||
| == Contributors | ||
|
|
||
| Alexander Flegontov, Intel + | ||
| Greg Lueck, Intel + | ||
| John Pennycook, Intel | ||
|
|
||
| == Dependencies | ||
|
|
||
| This extension is written against the SYCL 2020 specification, Revision 3. | ||
|
alexanderfle marked this conversation as resolved.
Outdated
|
||
|
|
||
| == Feature Test Macro | ||
|
|
||
| This extension provides a feature-test macro as described in the core SYCL | ||
| specification section 6.3.3 "Feature test macros". Therefore, an | ||
| implementation supporting this extension must predefine the macro | ||
| `SYCL_EXT_ONEAPI_DISCARD_QUEUE_EVENTS` to one of the values defined in the table below. | ||
| Applications can test for the existence of this macro to determine if the | ||
| implementation supports this feature, or applications can test the macro's | ||
| value to determine which of the extension's APIs the implementation supports. | ||
|
|
||
| [%header,cols="1,5"] | ||
| |=== | ||
| |Value |Description | ||
| |1 |Initial extension version. Base features are supported. | ||
| |=== | ||
|
|
||
| == Overview | ||
|
|
||
| This extension adds `ext::oneapi::property::queue::discard_events` property for `sycl::queue`, | ||
| by using this property the application informs a SYCL implementation that the returned event can be discarded. | ||
|
alexanderfle marked this conversation as resolved.
Outdated
|
||
|
|
||
| In cases where it will be possible, the implementation makes a decision about internal optimization -- avoiding | ||
| the creation of low-level event from the backend. The property affects only kernel cases, | ||
| i.e. avoid creating low-level event for submission of kernels. | ||
| Performance improvement is the main motivation for using this property. | ||
|
alexanderfle marked this conversation as resolved.
Outdated
|
||
|
|
||
| Below is a usage example: | ||
| [source,c++] | ||
| ---- | ||
| sycl::property_list props{ext::oneapi::property::queue::discard_events{} | ||
|
alexanderfle marked this conversation as resolved.
Outdated
|
||
| property::queue::in_order{}}; | ||
| sycl::queue Queue( props ); | ||
|
|
||
| // some USM preparations .. | ||
|
|
||
| sycl::event e1, e2, e3; | ||
|
|
||
| // returning "invalid" events from each submission function: | ||
| e1 = Queue.parallel_for(NDRange, [=](nd_item<1> item){ do_smth1(); }); | ||
|
|
||
| e2 = Queue.single_task([=](){ do_smth2(); }); | ||
|
|
||
| e3 = Queue.submit([&](handler &CGH) { CGH.parallel_for(NDRange, [=](nd_item<1> item){ do_smth3(); }); }); | ||
|
|
||
| Queue.wait(); | ||
| ---- | ||
|
|
||
| In the example above, the application doesn't use sycl events: `e1`, `e2`, `e3` | ||
| and is waiting for the end of work by `queue::wait()`. The returned events will be | ||
| _invalid_ events. _invalid_ event is a `sycl::event` with a status `invalid` and the event returns from each kernel | ||
| submission when queue is created with the `discard_events` property. | ||
|
alexanderfle marked this conversation as resolved.
Outdated
|
||
| See the description of behavior for this event below for details. | ||
|
|
||
| Here, only those methods for the _invalid_ event are described that have behavior different from the default event behavior: | ||
|
alexanderfle marked this conversation as resolved.
Outdated
|
||
| [source,c++] | ||
| ---- | ||
| // must throw an exception with the errc::invalid error code. | ||
| void wait(); | ||
|
|
||
| // if invalid event is passed into the function, must throw an exception with the errc::invalid error code. | ||
| static void wait(const std::vector<event> &eventList); | ||
|
|
||
| // must throw an exception with the errc::invalid error code. | ||
| void wait_and_throw(); | ||
|
|
||
| // if invalid event is passed into the function, must throw an exception with the errc::invalid error code. | ||
| static void wait_and_throw(const std::vector<event> &eventList); | ||
|
|
||
| // must return info::event_command_status::ext_oneapi_unknown | ||
| get_info<info::event::command_execution_status>() const; | ||
| ---- | ||
|
alexanderfle marked this conversation as resolved.
|
||
|
|
||
| The behavior when _invalid_ event is passed into handler API: | ||
| [source,c++] | ||
| ---- | ||
| // must throw an exception with the errc::invalid error code. | ||
| handler::depends_on(event Event) | ||
|
|
||
| // must throw an exception with the errc::invalid error code. | ||
| handler::depends_on(const std::vector<event> &Events) | ||
| ---- | ||
|
|
||
|
alexanderfle marked this conversation as resolved.
|
||
| == DPCPP implementation limitations | ||
|
|
||
| The `discard_events` property should be taken as a hint for SYCL implementation. | ||
| In all kernel cases, SYCL implementation returns _invalid_ event, regardless of whether the optimization was done. | ||
| Optimization occurs if the queue has `discard_events` property and all features presented below are not met: | ||
|
alexanderfle marked this conversation as resolved.
Outdated
|
||
|
|
||
| - Using out-of-order queue or using together with `enable_profiling` property | ||
| - Using https://github.com/intel/llvm/blob/sycl/sycl/doc/extensions/Assert/SYCL_ONEAPI_ASSERT.asciidoc[fallback assert feature] | ||
| - Using streams, buffer/image accessors (excluding local accessors) | ||
|
|
||
|
romanovvlad marked this conversation as resolved.
|
||
|
|
||
| See details for each below: | ||
|
|
||
| === Using out-of-order queue and/or using together with enable_profiling property | ||
|
|
||
| No optimization if a queue is created with the `discard_events` property and | ||
| the property list includes the `enable_profiling` property or does not include `in_order` property. | ||
|
|
||
| === Using fallback assert feature | ||
|
|
||
| No optimization if the application calls the `assert` macro from a command that is submitted to the queue unless | ||
| the device has native support for assertions (as specified by `aspect::ext_oneapi_native_assert`). | ||
|
|
||
| === Using streams, buffer/image accessors (excluding local accessors) | ||
|
|
||
| No optimization if a kernel that uses stream objects, buffer or image accessors is submitted to a queue created with | ||
| the `discard_events` property. But using local accessors does not affect optimization. | ||
|
|
||
| == Issues | ||
|
|
||
| None. | ||
|
|
||
| == Revision History | ||
|
|
||
| [cols="5,15,15,70"] | ||
| [grid="rows"] | ||
| [options="header"] | ||
| |======================================== | ||
| |Rev|Date|Author|Changes | ||
| |1|2021-11-09|Alexander Flegontov |*Initial public working draft* | ||
| |======================================== | ||
This file contains hidden or bidirectional Unicode text that may be interpreted or compiled differently than what appears below. To review, open the file in an editor that reveals hidden Unicode characters.
Learn more about bidirectional Unicode characters
Add this suggestion to a batch that can be applied as a single commit.
This suggestion is invalid because no changes were made to the code.
Suggestions cannot be applied while the pull request is closed.
Suggestions cannot be applied while viewing a subset of changes.
Only one suggestion per line can be applied in a batch.
Add this suggestion to a batch that can be applied as a single commit.
Applying suggestions on deleted lines is not supported.
You must change the existing code in this line in order to create a valid suggestion.
Outdated suggestions cannot be applied.
This suggestion has been applied or marked resolved.
Suggestions cannot be applied from pending reviews.
Suggestions cannot be applied on multi-line comments.
Suggestions cannot be applied while the pull request is queued to merge.
Suggestion cannot be applied right now. Please check back later.
Uh oh!
There was an error while loading. Please reload this page.